@@ -1,19 +1,20 @@
 /*!
- * Copyright (c) by XGBoost Contributors 2019-2021
+ * Copyright (c) by XGBoost Contributors 2019-2022
  */
 #ifndef XGBOOST_JSON_H_
 #define XGBOOST_JSON_H_
 
+#include <xgboost/intrusive_ptr.h>
 #include <xgboost/logging.h>
 #include <xgboost/parameter.h>
-#include <xgboost/intrusive_ptr.h>
+#include <xgboost/string_view.h>
 
+#include <functional>
 #include <map>
 #include <memory>
-#include <vector>
-#include <functional>
-#include <utility>
 #include <string>
+#include <utility>
+#include <vector>
 
 namespace xgboost {
 
@@ -38,7 +39,12 @@ class Value {
     kObject,  // std::map
     kArray,   // std::vector
     kBoolean,
-    kNull
+    kNull,
+    // typed array for ubjson
+    kNumberArray,
+    kU8Array,
+    kI32Array,
+    kI64Array
   };
 
   explicit Value(ValueKind _kind) : kind_{_kind} {}
@@ -46,13 +52,13 @@ class Value {
   ValueKind Type() const { return kind_; }
   virtual ~Value() = default;
 
-  virtual void Save(JsonWriter* writer) = 0;
+  virtual void Save(JsonWriter* writer) const = 0;
 
-  virtual Json& operator[](std::string const & key) = 0;
-  virtual Json& operator[](int ind) = 0;
+  virtual Json& operator[](std::string const& key);
+  virtual Json& operator[](int ind);
 
   virtual bool operator==(Value const& rhs) const = 0;
-  virtual Value& operator=(Value const& rhs) = 0;
+  virtual Value& operator=(Value const& rhs) = delete;
 
   std::string TypeStr() const;
 
@@ -83,21 +89,18 @@ class JsonString : public Value {
   JsonString(std::string const& str) :  // NOLINT
       Value(ValueKind::kString), str_{str} {}
   JsonString(std::string&& str) noexcept :  // NOLINT
-      Value(ValueKind::kString), str_{std::move(str)} {}
-  JsonString(JsonString&& str) noexcept :  // NOLINT
-      Value(ValueKind::kString), str_{std::move(str.str_)} {}
+      Value(ValueKind::kString), str_{std::forward<std::string>(str)} {}
+  JsonString(JsonString&& str) noexcept : Value(ValueKind::kString) {  // NOLINT
+    std::swap(str.str_, this->str_);
+  }
 
-  void Save(JsonWriter* writer) override;
-
-  Json& operator[](std::string const & key) override;
-  Json& operator[](int ind) override;
+  void Save(JsonWriter* writer) const override;
 
   std::string const& GetString() &&      { return str_; }
   std::string const& GetString() const & { return str_; }
   std::string&       GetString()       & { return str_; }
 
   bool operator==(Value const& rhs) const override;
-  Value& operator=(Value const& rhs) override;
 
   static bool IsClassOf(Value const* value) {
     return value->Type() == ValueKind::kString;
@@ -109,30 +112,78 @@ class JsonArray : public Value {
 
  public:
   JsonArray() : Value(ValueKind::kArray) {}
-  JsonArray(std::vector<Json>&& arr) noexcept :  // NOLINT
-      Value(ValueKind::kArray), vec_{std::move(arr)} {}
+  JsonArray(std::vector<Json>&& arr) noexcept  // NOLINT
+      : Value(ValueKind::kArray), vec_{std::forward<std::vector<Json>>(arr)} {}
   JsonArray(std::vector<Json> const& arr) :  // NOLINT
       Value(ValueKind::kArray), vec_{arr} {}
   JsonArray(JsonArray const& that) = delete;
   JsonArray(JsonArray && that) noexcept;
 
-  void Save(JsonWriter* writer) override;
+  void Save(JsonWriter* writer) const override;
 
-  Json& operator[](std::string const & key) override;
-  Json& operator[](int ind) override;
+  Json& operator[](int ind) override { return vec_.at(ind); }
+  // silent the partial oveeridden warning
+  Json& operator[](std::string const& key) override { return Value::operator[](key); }
 
   std::vector<Json> const& GetArray() &&      { return vec_; }
   std::vector<Json> const& GetArray() const & { return vec_; }
   std::vector<Json>&       GetArray()       & { return vec_; }
 
   bool operator==(Value const& rhs) const override;
-  Value& operator=(Value const& rhs) override;
 
   static bool IsClassOf(Value const* value) {
     return value->Type() == ValueKind::kArray;
   }
 };
 
+/**
+ * \brief Typed array for Universal Binary JSON.
+ *
+ * \tparam T The underlying primitive type.
+ * \tparam kind Value kind defined by JSON type.
+ */
+template <typename T, Value::ValueKind kind>
+class JsonTypedArray : public Value {
+  std::vector<T> vec_;
+
+ public:
+  using Type = T;
+
+  JsonTypedArray() : Value(kind) {}
+  explicit JsonTypedArray(size_t n) : Value(kind) { vec_.resize(n); }
+  JsonTypedArray(JsonTypedArray&& that) noexcept : Value{kind}, vec_{std::move(that.vec_)} {}
+
+  bool operator==(Value const& rhs) const override;
+
+  void Set(size_t i, T v) { vec_[i] = v; }
+  size_t Size() const { return vec_.size(); }
+
+  void Save(JsonWriter* writer) const override;
+
+  std::vector<T> const& GetArray() && { return vec_; }
+  std::vector<T> const& GetArray() const& { return vec_; }
+  std::vector<T>& GetArray() & { return vec_; }
+
+  static bool IsClassOf(Value const* value) { return value->Type() == kind; }
+};
+
+/**
+ * \brief Typed UBJSON array for 32-bit floating point.
+ */
+using F32Array = JsonTypedArray<float, Value::ValueKind::kNumberArray>;
+/**
+ * \brief Typed UBJSON array for uint8_t.
+ */
+using U8Array = JsonTypedArray<uint8_t, Value::ValueKind::kU8Array>;
+/**
+ * \brief Typed UBJSON array for int32_t.
+ */
+using I32Array = JsonTypedArray<int32_t, Value::ValueKind::kI32Array>;
+/**
+ * \brief Typed UBJSON array for int64_t.
+ */
+using I64Array = JsonTypedArray<int64_t, Value::ValueKind::kI64Array>;
+
 class JsonObject : public Value {
   std::map<std::string, Json> object_;
 
@@ -142,17 +193,17 @@ class JsonObject : public Value {
   JsonObject(JsonObject const& that) = delete;
   JsonObject(JsonObject && that) noexcept;
 
-  void Save(JsonWriter* writer) override;
+  void Save(JsonWriter* writer) const override;
 
-  Json& operator[](std::string const & key) override;
-  Json& operator[](int ind) override;
+  // silent the partial oveeridden warning
+  Json& operator[](int ind) override { return Value::operator[](ind); }
+  Json& operator[](std::string const& key) override { return object_[key]; }
 
   std::map<std::string, Json> const& GetObject() &&      { return object_; }
   std::map<std::string, Json> const& GetObject() const & { return object_; }
   std::map<std::string, Json> &      GetObject() &       { return object_; }
 
   bool operator==(Value const& rhs) const override;
-  Value& operator=(Value const& rhs) override;
 
   static bool IsClassOf(Value const* value) {
     return value->Type() == ValueKind::kObject;
@@ -181,18 +232,13 @@ class JsonNumber : public Value {
   JsonNumber(JsonNumber const& that) = delete;
   JsonNumber(JsonNumber&& that) noexcept : Value{ValueKind::kNumber}, number_{that.number_} {}
 
-  void Save(JsonWriter* writer) override;
-
-  Json& operator[](std::string const & key) override;
-  Json& operator[](int ind) override;
+  void Save(JsonWriter* writer) const override;
 
   Float const& GetNumber() &&      { return number_; }
   Float const& GetNumber() const & { return number_; }
   Float&       GetNumber()       & { return number_; }
 
-
   bool operator==(Value const& rhs) const override;
-  Value& operator=(Value const& rhs) override;
 
   static bool IsClassOf(Value const* value) {
     return value->Type() == ValueKind::kNumber;
@@ -230,16 +276,12 @@ class JsonInteger : public Value {
   JsonInteger(JsonInteger &&that) noexcept
       : Value{ValueKind::kInteger}, integer_{that.integer_} {}
 
-  Json& operator[](std::string const & key) override;
-  Json& operator[](int ind) override;
-
   bool operator==(Value const& rhs) const override;
-  Value& operator=(Value const& rhs) override;
 
   Int const& GetInteger() &&      { return integer_; }
   Int const& GetInteger() const & { return integer_; }
   Int& GetInteger() &             { return integer_; }
-  void Save(JsonWriter* writer) override;
+  void Save(JsonWriter* writer) const override;
 
   static bool IsClassOf(Value const* value) {
     return value->Type() == ValueKind::kInteger;
@@ -252,13 +294,9 @@ class JsonNull : public Value {
   JsonNull(std::nullptr_t) : Value(ValueKind::kNull) {}  // NOLINT
   JsonNull(JsonNull&&) noexcept : Value(ValueKind::kNull) {}
 
-  void Save(JsonWriter* writer) override;
-
-  Json& operator[](std::string const & key) override;
-  Json& operator[](int ind) override;
+  void Save(JsonWriter* writer) const override;
 
   bool operator==(Value const& rhs) const override;
-  Value& operator=(Value const& rhs) override;
 
   static bool IsClassOf(Value const* value) {
     return value->Type() == ValueKind::kNull;
@@ -281,60 +319,19 @@ class JsonBoolean : public Value {
   JsonBoolean(JsonBoolean&& value) noexcept:  // NOLINT
       Value(ValueKind::kBoolean), boolean_{value.boolean_} {}
 
-  void Save(JsonWriter* writer) override;
-
-  Json& operator[](std::string const & key) override;
-  Json& operator[](int ind) override;
+  void Save(JsonWriter* writer) const override;
 
   bool const& GetBoolean() &&      { return boolean_; }
   bool const& GetBoolean() const & { return boolean_; }
   bool&       GetBoolean()       & { return boolean_; }
 
   bool operator==(Value const& rhs) const override;
-  Value& operator=(Value const& rhs) override;
 
   static bool IsClassOf(Value const* value) {
     return value->Type() == ValueKind::kBoolean;
   }
 };
 
-struct StringView {
- private:
-  using CharT = char;  // unsigned char
-  using Traits = std::char_traits<CharT>;
-  CharT const* str_;
-  size_t size_;
-
- public:
-  StringView() = default;
-  StringView(CharT const* str, size_t size) : str_{str}, size_{size} {}
-  explicit StringView(std::string const& str): str_{str.c_str()}, size_{str.size()} {}
-  explicit StringView(CharT const* str) : str_{str}, size_{Traits::length(str)} {}
-
-  CharT const& operator[](size_t p) const { return str_[p]; }
-  CharT const& at(size_t p) const {  // NOLINT
-    CHECK_LT(p, size_);
-    return str_[p];
-  }
-  size_t size() const { return size_; }  // NOLINT
-  // Copies a portion of string.  Since we don't have std::from_chars and friends here, so
-  // copying substring is necessary for appending `\0`.  It's not too bad since string by
-  // default has small vector optimization, which is enabled by most if not all modern
-  // compilers for numeric values.
-  std::string substr(size_t beg, size_t n) const {  // NOLINT
-    CHECK_LE(beg, size_);
-    return std::string {str_ + beg, n < (size_ - beg) ? n : (size_ - beg)};
-  }
-  CharT const* c_str() const { return str_; }  // NOLINT
-
-  CharT const* cbegin() const { return str_; }         // NOLINT
-  CharT const* cend() const { return str_ + size(); }  // NOLINT
-  CharT const* begin() const { return str_; }          // NOLINT
-  CharT const* end() const { return str_ + size(); }   // NOLINT
-};
-
-std::ostream &operator<<(std::ostream &os, StringView const v);
-
 /*!
  * \brief Data structure representing JSON format.
  *
@@ -353,14 +350,22 @@ std::ostream &operator<<(std::ostream &os, StringView const v);
  * \endcode
  */
 class Json {
-  friend JsonWriter;
-
  public:
-  /*! \brief Load a Json object from string. */
-  static Json Load(StringView str);
+  /**
+   *  \brief Decode the JSON object.  Optional parameter mode for choosing between text
+   *         and binary (ubjson) input.
+   */
+  static Json Load(StringView str, std::ios::openmode mode = std::ios::in);
   /*! \brief Pass your own JsonReader. */
   static Json Load(JsonReader* reader);
-  static void Dump(Json json, std::string* out);
+  /**
+   *  \brief Encode the JSON object.  Optional parameter mode for choosing between text
+   *         and binary (ubjson) output.
+   */
+  static void Dump(Json json, std::string* out, std::ios::openmode mode = std::ios::out);
+  static void Dump(Json json, std::vector<char>* out, std::ios::openmode mode = std::ios::out);
+  /*! \brief Use your own JsonWriter. */
+  static void Dump(Json json, JsonWriter* writer);
 
   Json() : ptr_{new JsonNull} {}
 
@@ -370,34 +375,37 @@ class Json {
     ptr_.reset(new JsonNumber(std::move(number)));
     return *this;
   }
-
   // integer
   explicit Json(JsonInteger integer) : ptr_{new JsonInteger(std::move(integer))} {}
   Json& operator=(JsonInteger integer) {
     ptr_.reset(new JsonInteger(std::move(integer)));
     return *this;
   }
-
   // array
-  explicit Json(JsonArray list) :
-      ptr_ {new JsonArray(std::move(list))} {}
-  Json& operator=(JsonArray array) {
-    ptr_.reset(new JsonArray(std::move(array)));
+  explicit Json(JsonArray&& list) : ptr_{new JsonArray(std::forward<JsonArray>(list))} {}
+  Json& operator=(JsonArray&& array) {
+    ptr_.reset(new JsonArray(std::forward<JsonArray>(array)));
+    return *this;
+  }
+  // typed array
+  template <typename T, Value::ValueKind kind>
+  explicit Json(JsonTypedArray<T, kind>&& list)
+      : ptr_{new JsonTypedArray<T, kind>(std::forward<JsonTypedArray<T, kind>>(list))} {}
+  template <typename T, Value::ValueKind kind>
+  Json& operator=(JsonTypedArray<T, kind>&& array) {
+    ptr_.reset(new JsonTypedArray<T, kind>(std::forward<JsonTypedArray<T, kind>>(array)));
     return *this;
   }
-
   // object
-  explicit Json(JsonObject object) :
-      ptr_{new JsonObject(std::move(object))} {}
-  Json& operator=(JsonObject object) {
-    ptr_.reset(new JsonObject(std::move(object)));
+  explicit Json(JsonObject&& object) : ptr_{new JsonObject(std::forward<JsonObject>(object))} {}
+  Json& operator=(JsonObject&& object) {
+    ptr_.reset(new JsonObject(std::forward<JsonObject>(object)));
     return *this;
   }
   // string
-  explicit Json(JsonString str) :
-      ptr_{new JsonString(std::move(str))} {}
-  Json& operator=(JsonString str) {
-    ptr_.reset(new JsonString(std::move(str)));
+  explicit Json(JsonString&& str) : ptr_{new JsonString(std::forward<JsonString>(str))} {}
+  Json& operator=(JsonString&& str) {
+    ptr_.reset(new JsonString(std::forward<JsonString>(str)));
     return *this;
   }
   // bool
@@ -417,7 +425,7 @@ class Json {
 
   // copy
   Json(Json const& other) = default;
-  Json& operator=(Json const& other);
+  Json& operator=(Json const& other) = default;
   // move
   Json(Json &&other) noexcept { std::swap(this->ptr_, other.ptr_); }
   Json &operator=(Json &&other) noexcept {
@@ -446,18 +454,28 @@ class Json {
     return os;
   }
 
+  IntrusivePtr<Value> const& Ptr() const { return ptr_; }
+
  private:
   IntrusivePtr<Value> ptr_;
 };
 
+/**
+ * \brief Check whether a Json object has specific type.
+ *
+ * \code
+ *   Json json {Array{}};
+ *   bool is_array = IsA<Array>(json);
+ *   CHECK(is_array);
+ * \endcode
+ */
 template <typename T>
-bool IsA(Json const j) {
+bool IsA(Json const& j) {
   auto const& v = j.GetValue();
   return IsA<T>(&v);
 }
 
 namespace detail {
-
 // Number
 template <typename T,
           typename std::enable_if<
@@ -528,6 +546,16 @@ std::vector<Json> const& GetImpl(T& val) {  // NOLINT
   return val.GetArray();
 }
 
+// Typed Array
+template <typename T, Value::ValueKind kind>
+std::vector<T>& GetImpl(JsonTypedArray<T, kind>& val) {  // NOLINT
+  return val.GetArray();
+}
+template <typename T, Value::ValueKind kind>
+std::vector<T> const& GetImpl(JsonTypedArray<T, kind> const& val) {
+  return val.GetArray();
+}
+
 // Object
 template <typename T,
           typename std::enable_if<
@@ -541,7 +569,6 @@ template <typename T,
 std::map<std::string, Json> const& GetImpl(T& val) {  // NOLINT
   return val.GetObject();
 }
-
 }  // namespace detail
 
 /*!
