@@ -1,28 +1,31 @@
+# pylint: disable=too-many-instance-attributes, too-many-arguments, too-many-branches
 """
 This script is a variant of dmlc-core/dmlc_tracker/tracker.py,
 which is a specialized version for xgboost tasks.
 """
-
-# pylint: disable=invalid-name, missing-docstring, too-many-arguments, too-many-locals
-# pylint: disable=too-many-branches, too-many-statements, too-many-instance-attributes
 import socket
 import struct
-import time
 import logging
 from threading import Thread
 import argparse
 import sys
 
+from typing import Dict, List, Tuple, Union, Optional, Set
 
-class ExSocket(object):
+_RingMap = Dict[int, Tuple[int, int]]
+_TreeMap = Dict[int, List[int]]
+
+
+class ExSocket:
     """
     Extension of socket to handle recv and send of special data
     """
 
-    def __init__(self, sock):
+    def __init__(self, sock: socket.socket) -> None:
         self.sock = sock
 
-    def recvall(self, nbytes):
+    def recvall(self, nbytes: int) -> bytes:
+        """Receive number of bytes."""
         res = []
         nread = 0
         while nread < nbytes:
@@ -31,56 +34,81 @@ class ExSocket(object):
             res.append(chunk)
         return b''.join(res)
 
-    def recvint(self):
+    def recvint(self) -> int:
+        """Receive an integer of 32 bytes"""
         return struct.unpack('@i', self.recvall(4))[0]
 
-    def sendint(self, n):
-        self.sock.sendall(struct.pack('@i', n))
+    def sendint(self, value: int) -> None:
+        """Send an integer of 32 bytes"""
+        self.sock.sendall(struct.pack('@i', value))
 
-    def sendstr(self, s):
-        self.sendint(len(s))
-        self.sock.sendall(s.encode())
+    def sendstr(self, value: str) -> None:
+        """Send a Python string"""
+        self.sendint(len(value))
+        self.sock.sendall(value.encode())
 
-    def recvstr(self):
+    def recvstr(self) -> str:
+        """Receive a Python string"""
         slen = self.recvint()
         return self.recvall(slen).decode()
 
 
 # magic number used to verify existence of data
-kMagic = 0xff99
+MAGIC_NUM = 0xff99
 
 
-def get_some_ip(host):
+def get_some_ip(host: str) -> str:
+    """Get ip from host"""
     return socket.getaddrinfo(host, None)[0][4][0]
 
 
-def get_family(addr):
+def get_family(addr: str) -> int:
+    """Get network family from address."""
     return socket.getaddrinfo(addr, None)[0][0]
 
 
-class SlaveEntry(object):
-    def __init__(self, sock, s_addr):
-        slave = ExSocket(sock)
-        self.sock = slave
+class WorkerEntry:
+    """Hanlder to each worker."""
+    def __init__(self, sock: socket.socket, s_addr: Tuple[str, int]):
+        worker = ExSocket(sock)
+        self.sock = worker
         self.host = get_some_ip(s_addr[0])
-        magic = slave.recvint()
-        assert magic == kMagic, f"invalid magic number={magic} from {self.host}"
-        slave.sendint(kMagic)
-        self.rank = slave.recvint()
-        self.world_size = slave.recvint()
-        self.jobid = slave.recvstr()
-        self.cmd = slave.recvstr()
+        magic = worker.recvint()
+        assert magic == MAGIC_NUM, f"invalid magic number={magic} from {self.host}"
+        worker.sendint(MAGIC_NUM)
+        self.rank = worker.recvint()
+        self.world_size = worker.recvint()
+        self.jobid = worker.recvstr()
+        self.cmd = worker.recvstr()
         self.wait_accept = 0
-        self.port = None
+        self.port: Optional[int] = None
 
-    def decide_rank(self, job_map):
+    def print(self, use_logger: bool) -> None:
+        """Execute the print command from worker."""
+        msg = self.sock.recvstr()
+        # On dask we use print to avoid setting global verbosity.
+        if use_logger:
+            logging.info(msg.strip())
+        else:
+            print(msg.strip(), flush=True)
+
+    def decide_rank(self, job_map: Dict[str, int]) -> int:
+        """Get the rank of current entry."""
         if self.rank >= 0:
             return self.rank
         if self.jobid != 'NULL' and self.jobid in job_map:
             return job_map[self.jobid]
         return -1
 
-    def assign_rank(self, rank, wait_conn, tree_map, parent_map, ring_map):
+    def assign_rank(
+        self,
+        rank: int,
+        wait_conn: Dict[int, "WorkerEntry"],
+        tree_map: _TreeMap,
+        parent_map: Dict[int, int],
+        ring_map: _RingMap,
+    ) -> List[int]:
+        """Assign the rank for current entry."""
         self.rank = rank
         nnset = set(tree_map[rank])
         rprev, rnext = ring_map[rank]
@@ -105,6 +133,12 @@ class SlaveEntry(object):
             self.sock.sendint(rnext)
         else:
             self.sock.sendint(-1)
+
+        return self._get_remote(wait_conn, nnset)
+
+    def _get_remote(
+        self, wait_conn: Dict[int, "WorkerEntry"], nnset: Set[int]
+    ) -> List[int]:
         while True:
             ngood = self.sock.recvint()
             goodset = set([])
@@ -120,7 +154,9 @@ class SlaveEntry(object):
             self.sock.sendint(len(badset) - len(conset))
             for r in conset:
                 self.sock.sendstr(wait_conn[r].host)
-                self.sock.sendint(wait_conn[r].port)
+                port = wait_conn[r].port
+                assert port is not None
+                self.sock.sendint(port)
                 self.sock.sendint(r)
             nerr = self.sock.recvint()
             if nerr != 0:
@@ -138,13 +174,13 @@ class SlaveEntry(object):
             return rmset
 
 
-class RabitTracker(object):
+class RabitTracker:
     """
     tracker for rabit
     """
 
     def __init__(
-        self, hostIP, nslave, port=9091, port_end=9999, use_logger: bool = False
+        self, host_ip: str, n_workers: int, port: int = 0, use_logger: bool = False
     ) -> None:
         """A Python implementation of RABIT tracker.
 
@@ -155,67 +191,60 @@ class RabitTracker(object):
             function is used instead.
 
         """
-        sock = socket.socket(get_family(hostIP), socket.SOCK_STREAM)
-        for _port in range(port, port_end):
-            try:
-                sock.bind((hostIP, _port))
-                self.port = _port
-                break
-            except socket.error as e:
-                if e.errno in [98, 48]:
-                    continue
-                raise
+        sock = socket.socket(get_family(host_ip), socket.SOCK_STREAM)
+        sock.bind((host_ip, port))
+        self.port = sock.getsockname()[1]
         sock.listen(256)
         self.sock = sock
-        self.hostIP = hostIP
-        self.thread = None
-        self.start_time = None
-        self.end_time = None
-        self.nslave = nslave
+        self.host_ip = host_ip
+        self.thread: Optional[Thread] = None
+        self.n_workers = n_workers
         self._use_logger = use_logger
-        logging.info('start listen on %s:%d', hostIP, self.port)
+        logging.info("start listen on %s:%d", host_ip, self.port)
 
-    def __del__(self):
-        self.sock.close()
+    def __del__(self) -> None:
+        if hasattr(self, "sock"):
+            self.sock.close()
 
     @staticmethod
-    def get_neighbor(rank, nslave):
+    def _get_neighbor(rank: int, n_workers: int) -> List[int]:
         rank = rank + 1
         ret = []
         if rank > 1:
             ret.append(rank // 2 - 1)
-        if rank * 2 - 1 < nslave:
+        if rank * 2 - 1 < n_workers:
             ret.append(rank * 2 - 1)
-        if rank * 2 < nslave:
+        if rank * 2 < n_workers:
             ret.append(rank * 2)
         return ret
 
-    def slave_envs(self):
+    def worker_envs(self) -> Dict[str, Union[str, int]]:
         """
-        get enviroment variables for slaves
+        get environment variables for workers
         can be passed in as args or envs
         """
-        return {'DMLC_TRACKER_URI': self.hostIP,
-                'DMLC_TRACKER_PORT': self.port}
+        return {'DMLC_TRACKER_URI': self.host_ip, 'DMLC_TRACKER_PORT': self.port}
 
-    def get_tree(self, nslave):
-        tree_map = {}
-        parent_map = {}
-        for r in range(nslave):
-            tree_map[r] = self.get_neighbor(r, nslave)
+    def _get_tree(self, n_workers: int) -> Tuple[_TreeMap, Dict[int, int]]:
+        tree_map: _TreeMap = {}
+        parent_map: Dict[int, int] = {}
+        for r in range(n_workers):
+            tree_map[r] = self._get_neighbor(r, n_workers)
             parent_map[r] = (r + 1) // 2 - 1
         return tree_map, parent_map
 
-    def find_share_ring(self, tree_map, parent_map, r):
+    def find_share_ring(
+        self, tree_map: _TreeMap, parent_map: Dict[int, int], rank: int
+    ) -> List[int]:
         """
         get a ring structure that tends to share nodes with the tree
-        return a list starting from r
+        return a list starting from rank
         """
-        nset = set(tree_map[r])
-        cset = nset - set([parent_map[r]])
+        nset = set(tree_map[rank])
+        cset = nset - set([parent_map[rank]])
         if not cset:
-            return [r]
-        rlst = [r]
+            return [rank]
+        rlst = [rank]
         cnt = 0
         for v in cset:
             vlst = self.find_share_ring(tree_map, parent_map, v)
@@ -225,70 +254,66 @@ class RabitTracker(object):
             rlst += vlst
         return rlst
 
-    def get_ring(self, tree_map, parent_map):
+    def get_ring(self, tree_map: _TreeMap, parent_map: Dict[int, int]) -> _RingMap:
         """
         get a ring connection used to recover local data
         """
         assert parent_map[0] == -1
         rlst = self.find_share_ring(tree_map, parent_map, 0)
         assert len(rlst) == len(tree_map)
-        ring_map = {}
-        nslave = len(tree_map)
-        for r in range(nslave):
-            rprev = (r + nslave - 1) % nslave
-            rnext = (r + 1) % nslave
+        ring_map: _RingMap = {}
+        n_workers = len(tree_map)
+        for r in range(n_workers):
+            rprev = (r + n_workers - 1) % n_workers
+            rnext = (r + 1) % n_workers
             ring_map[rlst[r]] = (rlst[rprev], rlst[rnext])
         return ring_map
 
-    def get_link_map(self, nslave):
+    def get_link_map(self, n_workers: int) -> Tuple[_TreeMap, Dict[int, int], _RingMap]:
         """
         get the link map, this is a bit hacky, call for better algorithm
         to place similar nodes together
         """
-        tree_map, parent_map = self.get_tree(nslave)
+        tree_map, parent_map = self._get_tree(n_workers)
         ring_map = self.get_ring(tree_map, parent_map)
         rmap = {0: 0}
         k = 0
-        for i in range(nslave - 1):
+        for i in range(n_workers - 1):
             k = ring_map[k][1]
             rmap[k] = i + 1
 
-        ring_map_ = {}
-        tree_map_ = {}
-        parent_map_ = {}
+        ring_map_: _RingMap = {}
+        tree_map_: _TreeMap = {}
+        parent_map_: Dict[int, int] = {}
         for k, v in ring_map.items():
             ring_map_[rmap[k]] = (rmap[v[0]], rmap[v[1]])
-        for k, v in tree_map.items():
-            tree_map_[rmap[k]] = [rmap[x] for x in v]
-        for k, v in parent_map.items():
+        for k, tree_nodes in tree_map.items():
+            tree_map_[rmap[k]] = [rmap[x] for x in tree_nodes]
+        for k, parent in parent_map.items():
             if k != 0:
-                parent_map_[rmap[k]] = rmap[v]
+                parent_map_[rmap[k]] = rmap[parent]
             else:
                 parent_map_[rmap[k]] = -1
         return tree_map_, parent_map_, ring_map_
 
-    def accept_slaves(self, nslave):
-        # set of nodes that finishs the job
-        shutdown = {}
+    def accept_workers(self, n_workers: int) -> None:
+        """Wait for all workers to connect to the tracker."""
+        # set of nodes that finishes the job
+        shutdown: Dict[int, WorkerEntry] = {}
         # set of nodes that is waiting for connections
-        wait_conn = {}
+        wait_conn: Dict[int, WorkerEntry] = {}
         # maps job id to rank
-        job_map = {}
+        job_map: Dict[str, int] = {}
         # list of workers that is pending to be assigned rank
-        pending = []
+        pending: List[WorkerEntry] = []
         # lazy initialize tree_map
         tree_map = None
 
-        while len(shutdown) != nslave:
+        while len(shutdown) != n_workers:
             fd, s_addr = self.sock.accept()
-            s = SlaveEntry(fd, s_addr)
+            s = WorkerEntry(fd, s_addr)
             if s.cmd == 'print':
-                msg = s.sock.recvstr()
-                # On dask we use print to avoid setting global verbosity.
-                if self._use_logger:
-                    logging.info(msg.strip())
-                else:
-                    print(msg.strip(), flush=True)
+                s.print(self._use_logger)
                 continue
             if s.cmd == 'shutdown':
                 assert s.rank >= 0 and s.rank not in shutdown
@@ -297,16 +322,16 @@ class RabitTracker(object):
                 logging.debug('Received %s signal from %d', s.cmd, s.rank)
                 continue
             assert s.cmd in ("start", "recover")
-            # lazily initialize the slaves
+            # lazily initialize the workers
             if tree_map is None:
                 assert s.cmd == 'start'
                 if s.world_size > 0:
-                    nslave = s.world_size
-                tree_map, parent_map, ring_map = self.get_link_map(nslave)
+                    n_workers = s.world_size
+                tree_map, parent_map, ring_map = self.get_link_map(n_workers)
                 # set of nodes that is pending for getting up
-                todo_nodes = list(range(nslave))
+                todo_nodes = list(range(n_workers))
             else:
-                assert s.world_size in (-1, nslave)
+                assert s.world_size in (-1, n_workers)
             if s.cmd == 'recover':
                 assert s.rank >= 0
 
@@ -327,57 +352,62 @@ class RabitTracker(object):
                         logging.debug('Received %s signal from %s; assign rank %d',
                                       s.cmd, s.host, s.rank)
                 if not todo_nodes:
-                    logging.info('@tracker All of %d nodes getting started', nslave)
-                    self.start_time = time.time()
+                    logging.info('@tracker All of %d nodes getting started', n_workers)
             else:
                 s.assign_rank(rank, wait_conn, tree_map, parent_map, ring_map)
                 logging.debug('Received %s signal from %d', s.cmd, s.rank)
                 if s.wait_accept > 0:
                     wait_conn[rank] = s
         logging.info('@tracker All nodes finishes job')
-        self.end_time = time.time()
-        logging.info('@tracker %s secs between node start and job finish',
-                     str(self.end_time - self.start_time))
 
-    def start(self, nslave):
-        def run():
-            self.accept_slaves(nslave)
+    def start(self, n_workers: int) -> None:
+        """Strat the tracker, it will wait for `n_workers` to connect."""
+        def run() -> None:
+            self.accept_workers(n_workers)
 
         self.thread = Thread(target=run, args=(), daemon=True)
         self.thread.start()
 
-    def join(self):
-        while self.thread.is_alive():
+    def join(self) -> None:
+        """Wait for the tracker to finish."""
+        while self.thread is not None and self.thread.is_alive():
             self.thread.join(100)
 
-    def alive(self):
-        return self.thread.is_alive()
+    def alive(self) -> bool:
+        """Wether the tracker thread is alive"""
+        return self.thread is not None and self.thread.is_alive()
 
 
-def get_host_ip(hostIP=None):
-    if hostIP is None or hostIP == 'auto':
-        hostIP = 'ip'
+def get_host_ip(host_ip: Optional[str] = None) -> str:
+    """Get the IP address of current host.  If `host_ip` is not none then it will be
+    returned as it's
 
-    if hostIP == 'dns':
-        hostIP = socket.getfqdn()
-    elif hostIP == 'ip':
+    """
+    if host_ip is None or host_ip == 'auto':
+        host_ip = 'ip'
+
+    if host_ip == 'dns':
+        host_ip = socket.getfqdn()
+    elif host_ip == 'ip':
         from socket import gaierror
         try:
-            hostIP = socket.gethostbyname(socket.getfqdn())
+            host_ip = socket.gethostbyname(socket.getfqdn())
         except gaierror:
             logging.debug(
                 'gethostbyname(socket.getfqdn()) failed... trying on hostname()'
             )
-            hostIP = socket.gethostbyname(socket.gethostname())
-        if hostIP.startswith("127."):
+            host_ip = socket.gethostbyname(socket.gethostname())
+        if host_ip.startswith("127."):
             s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
             # doesn't have to be reachable
             s.connect(('10.255.255.255', 1))
-            hostIP = s.getsockname()[0]
-    return hostIP
+            host_ip = s.getsockname()[0]
+
+    assert host_ip is not None
+    return host_ip
 
 
-def start_rabit_tracker(args):
+def start_rabit_tracker(args: argparse.Namespace) -> None:
     """Standalone function to start rabit tracker.
 
     Parameters
@@ -386,9 +416,9 @@ def start_rabit_tracker(args):
     """
     envs = {"DMLC_NUM_WORKER": args.num_workers, "DMLC_NUM_SERVER": args.num_servers}
     rabit = RabitTracker(
-        hostIP=get_host_ip(args.host_ip), nslave=args.num_workers, use_logger=True
+        host_ip=get_host_ip(args.host_ip), n_workers=args.num_workers, use_logger=True
     )
-    envs.update(rabit.slave_envs())
+    envs.update(rabit.worker_envs())
     rabit.start(args.num_workers)
     sys.stdout.write("DMLC_TRACKER_ENV_START\n")
     # simply write configuration to stdout
@@ -399,13 +429,15 @@ def start_rabit_tracker(args):
     rabit.join()
 
 
-def main():
+def main() -> None:
     """Main function if tracker is executed in standalone mode."""
     parser = argparse.ArgumentParser(description='Rabit Tracker start.')
     parser.add_argument('--num-workers', required=True, type=int,
-                        help='Number of worker proccess to be launched.')
-    parser.add_argument('--num-servers', default=0, type=int,
-                        help='Number of server process to be launched. Only used in PS jobs.')
+                        help='Number of worker process to be launched.')
+    parser.add_argument(
+        '--num-servers', default=0, type=int,
+        help='Number of server process to be launched. Only used in PS jobs.'
+    )
     parser.add_argument('--host-ip', default=None, type=str,
                         help=('Host IP addressed, this is only needed ' +
                               'if the host IP cannot be automatically guessed.'))
