@@ -91,6 +91,12 @@ xgboost::bst_float GetMetricEval(
   std::vector<xgboost::bst_float> weights = std::vector<xgboost::bst_float>(),
   std::vector<xgboost::bst_uint> groups = std::vector<xgboost::bst_uint>());
 
+double GetMultiMetricEval(xgboost::Metric* metric,
+                          xgboost::HostDeviceVector<xgboost::bst_float> const& preds,
+                          xgboost::linalg::Tensor<float, 2> const& labels,
+                          std::vector<xgboost::bst_float> weights = {},
+                          std::vector<xgboost::bst_uint> groups = {});
+
 namespace xgboost {
 bool IsNear(std::vector<xgboost::bst_float>::const_iterator _beg1,
             std::vector<xgboost::bst_float>::const_iterator _end1,
@@ -106,42 +112,39 @@ bool IsNear(std::vector<xgboost::bst_float>::const_iterator _beg1,
  */
 class SimpleLCG {
  private:
-  using StateType = int64_t;
+  using StateType = uint64_t;
   static StateType constexpr kDefaultInit = 3;
-  static StateType constexpr default_alpha_ = 61;
-  static StateType constexpr max_value_ = ((StateType)1 << 32) - 1;
+  static StateType constexpr kDefaultAlpha = 61;
+  static StateType constexpr kMaxValue = (static_cast<StateType>(1) << 32) - 1;
 
   StateType state_;
   StateType const alpha_;
   StateType const mod_;
 
-  StateType seed_;
+ public:
+  using result_type = StateType;  // NOLINT
 
  public:
-  SimpleLCG() : state_{kDefaultInit},
-                alpha_{default_alpha_}, mod_{max_value_}, seed_{state_}{}
+  SimpleLCG() : state_{kDefaultInit}, alpha_{kDefaultAlpha}, mod_{kMaxValue} {}
   SimpleLCG(SimpleLCG const& that) = default;
   SimpleLCG(SimpleLCG&& that) = default;
 
-  void Seed(StateType seed) {
-    seed_ = seed;
-  }
+  void Seed(StateType seed) { state_ = seed % mod_; }
   /*!
    * \brief Initialize SimpleLCG.
    *
    * \param state  Initial state, can also be considered as seed. If set to
    *               zero, SimpleLCG will use internal default value.
-   * \param alpha  multiplier
-   * \param mod    modulo
    */
-  explicit SimpleLCG(StateType state,
-                     StateType alpha=default_alpha_, StateType mod=max_value_)
-      : state_{state == 0 ? kDefaultInit : state},
-        alpha_{alpha}, mod_{mod} , seed_{state} {}
+  explicit SimpleLCG(StateType state)
+      : state_{state == 0 ? kDefaultInit : state}, alpha_{kDefaultAlpha}, mod_{kMaxValue} {}
 
   StateType operator()();
   StateType Min() const;
   StateType Max() const;
+
+  constexpr result_type static min() { return 0; };         // NOLINT
+  constexpr result_type static max() { return kMaxValue; }  // NOLINT
 };
 
 template <typename ResultT>
@@ -191,6 +194,7 @@ Json GetArrayInterface(HostDeviceVector<T> *storage, size_t rows, size_t cols) {
   if (storage->DeviceCanRead()) {
     array_interface["data"][0] =
         Integer(reinterpret_cast<int64_t>(storage->ConstDevicePointer()));
+    array_interface["stream"] = nullptr;
   } else {
     array_interface["data"][0] =
         Integer(reinterpret_cast<int64_t>(storage->ConstHostPointer()));
@@ -201,9 +205,9 @@ Json GetArrayInterface(HostDeviceVector<T> *storage, size_t rows, size_t cols) {
   array_interface["shape"][0] = rows;
   array_interface["shape"][1] = cols;
 
-  char t = ArrayInterfaceHandler::TypeChar<T>();
+  char t = linalg::detail::ArrayInterfaceHandler::TypeChar<T>();
   array_interface["typestr"] = String(std::string{"<"} + t + std::to_string(sizeof(T)));
-  array_interface["version"] = 1;
+  array_interface["version"] = 3;
   return array_interface;
 }
 
@@ -217,10 +221,12 @@ class RandomDataGenerator {
   float upper_;
 
   int32_t device_;
-  int32_t seed_;
+  uint64_t seed_;
   SimpleLCG lcg_;
 
   size_t bins_;
+  std::vector<FeatureType> ft_;
+  bst_cat_t max_cat_;
 
   Json ArrayInterfaceImpl(HostDeviceVector<float> *storage, size_t rows,
                           size_t cols) const;
@@ -242,7 +248,7 @@ class RandomDataGenerator {
     device_ = d;
     return *this;
   }
-  RandomDataGenerator& Seed(int32_t s) {
+  RandomDataGenerator& Seed(uint64_t s) {
     seed_ = s;
     lcg_.Seed(seed_);
     return *this;
@@ -251,6 +257,16 @@ class RandomDataGenerator {
     bins_ = b;
     return *this;
   }
+  RandomDataGenerator& Type(common::Span<FeatureType> ft) {
+    CHECK_EQ(ft.size(), cols_);
+    ft_.resize(ft.size());
+    std::copy(ft.cbegin(), ft.cend(), ft_.begin());
+    return *this;
+  }
+  RandomDataGenerator& MaxCategory(bst_cat_t cat) {
+    max_cat_ = cat;
+    return *this;
+  }
 
   void GenerateDense(HostDeviceVector<float>* out) const;
 
@@ -302,11 +318,26 @@ GenerateRandomCategoricalSingleColumn(int n, size_t num_categories) {
 std::shared_ptr<DMatrix> GetDMatrixFromData(const std::vector<float> &x,
                                             int num_rows, int num_columns);
 
+/**
+ * \brief Create Sparse Page using data iterator.
+ *
+ * \param n_samples  Total number of rows for all batches combined.
+ * \param n_features Number of features
+ * \param n_batches  Number of batches
+ * \param prefix     Cache prefix, can be used for specifying file path.
+ *
+ * \return A Sparse DMatrix with n_batches.
+ */
+std::unique_ptr<DMatrix> CreateSparsePageDMatrix(bst_row_t n_samples, bst_feature_t n_features,
+                                                 size_t n_batches, std::string prefix = "cache");
+
+/**
+ * Deprecated, stop using it
+ */
 std::unique_ptr<DMatrix> CreateSparsePageDMatrix(size_t n_entries, std::string prefix = "cache");
 
 /**
- * \fn std::unique_ptr<DMatrix> CreateSparsePageDMatrixWithRC(size_t n_rows, size_t n_cols,
- *                                                            size_t page_size);
+ * Deprecated, stop using it
  *
  * \brief Creates dmatrix with some records, each record containing random number of
  *        features in [1, n_cols]
@@ -326,7 +357,8 @@ std::unique_ptr<DMatrix> CreateSparsePageDMatrixWithRC(
     size_t n_rows, size_t n_cols, size_t page_size, bool deterministic,
     const dmlc::TemporaryDirectory& tempdir = dmlc::TemporaryDirectory());
 
-gbm::GBTreeModel CreateTestModel(LearnerModelParam const* param, size_t n_classes = 1);
+gbm::GBTreeModel CreateTestModel(LearnerModelParam const* param, GenericParameter const* ctx,
+                                 size_t n_classes = 1);
 
 std::unique_ptr<GradientBooster> CreateTrainedGBM(
     std::string name, Args kwargs, size_t kRows, size_t kCols,
