@@ -7,6 +7,7 @@ sys.path.append("tests/python")
 # Don't import the test class, otherwise they will run twice.
 import test_callback as test_cb  # noqa
 import test_basic_models as test_bm
+import testing as tm
 rng = np.random.RandomState(1994)
 
 
@@ -14,16 +15,12 @@ class TestGPUBasicModels:
     cpu_test_cb = test_cb.TestCallbacks()
     cpu_test_bm = test_bm.TestModels()
 
-    def run_cls(self, X, y, deterministic):
-        cls = xgb.XGBClassifier(tree_method='gpu_hist',
-                                deterministic_histogram=deterministic,
-                                single_precision_histogram=True)
+    def run_cls(self, X, y):
+        cls = xgb.XGBClassifier(tree_method='gpu_hist', single_precision_histogram=True)
         cls.fit(X, y)
         cls.get_booster().save_model('test_deterministic_gpu_hist-0.json')
 
-        cls = xgb.XGBClassifier(tree_method='gpu_hist',
-                                deterministic_histogram=deterministic,
-                                single_precision_histogram=True)
+        cls = xgb.XGBClassifier(tree_method='gpu_hist', single_precision_histogram=True)
         cls.fit(X, y)
         cls.get_booster().save_model('test_deterministic_gpu_hist-1.json')
 
@@ -41,8 +38,7 @@ class TestGPUBasicModels:
         self.cpu_test_bm.run_custom_objective("gpu_hist")
 
     def test_eta_decay_gpu_hist(self):
-        self.cpu_test_cb.run_eta_decay('gpu_hist', True)
-        self.cpu_test_cb.run_eta_decay('gpu_hist', False)
+        self.cpu_test_cb.run_eta_decay('gpu_hist')
 
     def test_deterministic_gpu_hist(self):
         kRows = 1000
@@ -50,19 +46,22 @@ class TestGPUBasicModels:
         kClasses = 4
         # Create large values to force rounding.
         X = np.random.randn(kRows, kCols) * 1e4
-        y = np.random.randint(0, kClasses, size=kRows) * 1e4
+        y = np.random.randint(0, kClasses, size=kRows)
 
-        model_0, model_1 = self.run_cls(X, y, True)
+        model_0, model_1 = self.run_cls(X, y)
         assert model_0 == model_1
 
+    @pytest.mark.skipif(**tm.no_sklearn())
     def test_invalid_gpu_id(self):
-        X = np.random.randn(10, 5) * 1e4
-        y = np.random.randint(0, 2, size=10) * 1e4
+        from sklearn.datasets import load_digits
+        X, y = load_digits(return_X_y=True)
         # should pass with invalid gpu id
         cls1 = xgb.XGBClassifier(tree_method='gpu_hist', gpu_id=9999)
         cls1.fit(X, y)
         # should throw error with fail_on_invalid_gpu_id enabled
-        cls2 = xgb.XGBClassifier(tree_method='gpu_hist', gpu_id=9999, fail_on_invalid_gpu_id=True)
+        cls2 = xgb.XGBClassifier(
+            tree_method='gpu_hist', gpu_id=9999, fail_on_invalid_gpu_id=True
+        )
         try:
             cls2.fit(X, y)
             assert False, "Should have failed with with fail_on_invalid_gpu_id enabled"
